require_relative '../../lib/bin_diesel'
require_relative '../test_helper'

describe BinDiesel do
  describe "help flag" do
    it 'is equivalent to call -h and --help' do
      vincent = Class.new(TestDiesel)

      help_output = capture_std_out do
        swallow_exit { vincent.new(['--help']).run }
      end

      h_output = capture_std_out do
        swallow_exit { vincent.new(['-h']).run }
      end

      help_output.must_equal(h_output)
    end

    it 'swallows other options if help is called' do
      vincent = Class.new(TestDiesel) do
        opts_banner "This is my banner"

        opts_on '-e', '--error', 'Raise an error' do
          options.error = "Raised error"
        end

        run do
          puts options.error
        end
      end

      swallow_exit do
        lambda { vincent.new(['--error', '--help']).run }.must_output "This is my banner"
      end
    end

    it 'raises a system exit' do
      vincent = Class.new(TestDiesel)
      swallow_std_out do
        lambda{ vincent.new(["--help"]).run }.must_raise(SystemExit)
      end
    end

    it 'is not required' do
      vincent = Class.new(TestDiesel) do
        opts_banner "And here is my banner"
      end

      swallow_exit do
        lambda{ vincent.new.run }.must_output("-h, --help")
      end
    end

    describe "opts_banner" do
      it 'prints specified text on the first line when using a help flag' do
        swallow_exit do
          lambda{TestDiesel.new(["--help"]).run}.must_output("And here is my banner")
        end
      end
    end
  end

  describe "opts_description" do
    it 'prints in help text' do
      vincent = Class.new(TestDiesel) do
        opts_description "My description"
      end

      swallow_exit do
        lambda { vincent.new(['--help']).run }.must_output("My description")
      end
    end

    it 'prints after banner in help text' do
      vincent = Class.new(TestDiesel) do
        opts_banner "The banner"
        opts_description "The description"
      end

      swallow_exit do
        lambda { vincent.new(['--help']).run }.must_output("The banner\n\nThe description")
      end
    end

    it 'is not required' do
      vincent = Class.new(TestDiesel) do
        opts_banner "My banner"
      end

      swallow_exit do
        lambda { vincent.new(['--help']).run }.must_output("My banner")
      end
    end

    it 'prints in order specified, on separate lines' do
      vincent = Class.new(TestDiesel) do
        opts_banner "My Banner"

        opts_description "description 1"
        opts_description "description 2"
      end

      swallow_exit do
        lambda { vincent.new(['--help']).run }.must_output("description 1\ndescription 2")
      end
    end
  end

  describe "opts_required" do
    it 'raises an error if option not given during execution' do
      vincent = Class.new(TestDiesel) do
        opts_on '-r', '--required OPTION', 'A required option' do |value|
          options.required = value
        end

        opts_required :required
      end

      swallow_exit do
        lambda { vincent.new([]) }.must_output("!! missing argument: --required - Run with --help for help")
      end
    end

    it 'allows for more than one required option at a time' do
      vincent = Class.new(TestDiesel) do
        opts_on '-r', '--required OPTION', 'A required option' do |value|
          options.required = value
        end

        opts_on '-o', '--option OPTION', 'Another required option' do |value|
          options.option = value
        end

        opts_required :required, :option
      end

      swallow_exit do
        -> { vincent.new([]) }.must_output "!! missing argument: --required"
        -> { vincent.new(['--required', 'test']) }.must_output "!! missing argument: --option"
      end
    end

    it 'allows for more than one required option, specified in different calls' do
      vincent = Class.new(TestDiesel) do
        opts_on '-r', '--required OPTION', 'A required option' do |value|
          options.required = value
        end

        opts_on '-o', '--option OPTION', 'Another required option' do |value|
          options.option = value
        end

        opts_required :required
        opts_required :option
      end

      swallow_exit do
        -> { vincent.new([]) }.must_output "!! missing argument: --required"
        -> { vincent.new(['--required', 'test']) }.must_output "!! missing argument: --option"
      end
    end
  end

  describe "opts_accessor" do
    it 'sets an attribute_reader for the given option' do
      vincent = Class.new(TestDiesel) do
        opts_on '-o', '--option [OPTION]', 'An option' do |value|
          options.option = value
        end

        opts_accessor :option
      end

      swallow_exit do
        vincent.new(['--option', 'value']).option.must_equal('value')
      end
    end

    it 'allows for more than one option accessor at one time' do
      vincent = Class.new(TestDiesel) do
        opts_on '-o', '--option [OPTION]', 'An option' do |value|
          options.option = value
        end

        opts_on '-a', '--another-option [OPTION]', 'Another option' do |value|
          options.another_option = value
        end

        opts_accessor :option, :another_option
      end

      swallow_exit do
        vincent.new(['--option', 'value']).option.must_equal('value')
        vincent.new(['--another-option', 'value']).another_option.must_equal('value')
      end
    end

    it 'allows for more than one option accessor, specified in different calls' do
      vincent = Class.new(TestDiesel) do
        opts_on '-o', '--option [OPTION]', 'An option' do |value|
          options.option = value
        end

        opts_on '-a', '--another-option [OPTION]', 'Another option' do |value|
          options.another_option = value
        end

        opts_accessor :option
        opts_accessor :another_option
      end

      swallow_exit do
        vincent.new(['--option', 'value']).option.must_equal('value')
        vincent.new(['--another-option', 'value']).another_option.must_equal('value')
      end
    end

  end

  describe "opts_on" do
    it 'outputs help for the option with the --help flag' do
      vincent = Class.new(TestDiesel) do
        opts_on '-f', '--flag', 'A boolean flag' do |value|
          options.flag = value
        end
      end

      swallow_exit do
        lambda { vincent.new(['--help']).run }.must_output("A boolean flag")
      end
    end

    it 'allows boolean options' do
      vincent = Class.new(TestDiesel) do
        opts_on '-b', '--boolean-flag', 'A boolean flag' do |value|
          options.flag = value
        end

        run do
          puts "flag: #{flag}"
        end
      end

      swallow_exit do
        vbomb = vincent.new(['--boolean-flag'])
        vbomb.instance_variable_get(:@options).flag.must_equal(true)
      end
    end

    it 'allows options with user supplied values' do
      vincent = Class.new(TestDiesel) do
        opts_on '-u', '--user-specified [OPTION]', 'A user specified option' do |value|
          options.user_specified = value
        end

        run do
          puts "user_specified: #{options.user_specified}"
        end
      end

      swallow_exit do
        vbomb = vincent.new(['--user-specified', 'option'])
        vbomb.instance_variable_get(:@options).user_specified.must_equal('option')
      end
    end

    it 'accepts a short flag' do
      vincent = Class.new(TestDiesel) do
        opts_on '-s', '--short [OPTION]', 'A short option' do |value|
          options.short = value
        end
      end

      swallow_exit do
        vbomb = vincent.new(['-s', 'short'])
        vbomb.instance_variable_get(:@options).short.must_equal('short')
      end
    end

    it 'accepts a long flag' do
      vincent = Class.new(TestDiesel) do
        opts_on '-l', '--long [OPTION]', 'A long flag' do |value|
          options.long = value
        end
      end

      swallow_exit do
        vbomb = vincent.new(['--long', 'long'])
        vbomb.instance_variable_get(:@options).long.must_equal('long')
      end
    end
  end

  describe "dry_run flag" do
    it 'sets the `dry_run` var to true' do
      vincent = Class.new(TestDiesel)

      swallow_exit do
        vincent.new(['--dry-run']).dry_run.must_equal(true)
      end
    end
  end

  describe "verbose flag" do
    it 'sets the `verbose` var to true' do
      swallow_exit do
        TestDiesel.new(['--verbose']).verbose.must_equal(true)
      end
    end

    it 'allows info messages to be printed' do
      vincent = Class.new(TestDiesel) do
        run do
          info_message "An info message"
        end
      end

      swallow_exit do
        -> { vincent.new(['--verbose']).run }.must_output("** An info message\n")
      end
    end

    it 'allows messages from `message` to be printed' do
      vincent = Class.new(TestDiesel) do
        run do
          message "A message"
        end
      end

      swallow_exit do
        -> { vincent.new(['--verbose']).run }.must_output("A message\n")
      end
    end
  end

  describe "post_initialize" do
    it 'runs before `run`' do
      vincent = Class.new(TestDiesel) do
        post_initialize do
          message "post-initialize"
        end

        run do
          message "run"
        end
      end

      swallow_exit do
        -> { vincent.new(['--verbose']).run }.must_output("post-initialize\nrun\n")
      end
    end
  end

  describe "message" do
    it 'prints messages when verbose is true' do
      vincent = Class.new(TestDiesel) do
        run do
          message "A message"
        end
      end

      swallow_exit do
        -> { vincent.new(['--verbose']).run }.must_output("A message\n")
      end
    end

    it 'does not print messages when verbose is false' do
      vincent = Class.new(TestDiesel) do
        run do
          message "A message"
        end
      end

      output = capture_std_out do
        swallow_exit { vincent.new([]).run }
      end

      output.to_s.wont_include("A message")
    end
  end

  describe "info_message" do
    it 'prepends messages with `**`' do
      vincent = Class.new(TestDiesel) do
        run do
          info_message("An info message")
        end
      end

      swallow_exit do
        -> { vincent.new(['--verbose']).run }.must_output("** An info message\n")
      end
    end

    it 'does not print messages when verbose is false' do
      vincent = Class.new(TestDiesel) do
        run do
          info_message("An info message")
        end
      end

      output = capture_std_out do
        swallow_exit { vincent.new([]).run }
      end

      output.to_s.wont_include("** An info message")
    end

    it 'prints messages when verbose is true' do
      vincent = Class.new(TestDiesel) do
        run do
          info_message "An info message"
        end
      end

      swallow_exit do
        -> { vincent.new(['--verbose']).run }.must_output("** An info message\n")
      end
    end
  end

  describe "error_message" do
    it 'prepends messages with `!!`' do
      vincent = Class.new(TestDiesel) do
        run do
          error_message("An error message")
        end
      end

      swallow_exit do
        -> { vincent.new(['--verbose']).run }.must_output("!! An error message\n")
      end
    end

    it 'prints messages regardless of `verbose`' do
      vincent = Class.new(TestDiesel) do
        run do
          error_message("An error message")
        end
      end

      swallow_exit do
        -> { vincent.new([]).run }.must_output("!! An error message\n")
      end
    end
  end

  describe "dry_run?" do
    it 'returns true if the dry-run option is set' do
      swallow_exit do
        TestDiesel.new(['--dry-run']).dry_run?.must_equal(true)
      end
    end

    it 'returns false if the dry-run option is not set' do
      swallow_exit do
        TestDiesel.new([]).dry_run?.must_equal(false)
      end
    end
  end
end
